package ca.canadapost.cpcdp.ncshipping;

import ca.canadapost.cpcdp.ncshipping.stub.artifact.*;
import ca.canadapost.cpcdp.ncshipping.util.ClientUtil;
import org.apache.commons.codec.binary.Base64;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.io.FileOutputStream;
import java.util.Map;

/**
 * Sample code for the GetArtifact Canada Post service.
 * <p>
 * The GetArtifact service is used to retrieve a pdf of the shipping label
 * created by a prior Create Non-Contract Shipment call.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetArtifactClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_NCShipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            ArtifactService service = new ArtifactService();
            ArtifactPortType port = service.getArtifactPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/artifact");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetArtifactRequest request = new GetArtifactRequest();
            request.setLocale("FR");
            request.setArtifactId(2638223);

            //execute request
            GetArtifactResponse response = port.getArtifact(request);
            if(response.getMessages() == null){
                // decoding base64 certificate to a file
                String fileName = "ncShipmentArtifact.pdf";
                String encodedImage = response.getArtifactData().getImage();
                System.out.println("Base64 Encoded: " + encodedImage);
                System.out.println("Mime type: " + response.getArtifactData().getMimeType());

                FileOutputStream fos = new FileOutputStream(fileName);
                Base64 decoder = new Base64();
                byte[] decodedBytes = decoder.decode(encodedImage.getBytes());
                fos.write(decodedBytes);
                fos.close();

                System.out.println("Decoding to " + System.getProperty("user.dir") + System.getProperty("file.separator") + fileName);
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
