package ca.canadapost.cpcdp.ncshipping;

import ca.canadapost.cpcdp.ncshipping.stub.ncshipment.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.ncshipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Map;


/**
 * Sample code for the GetNCShipmentPublicKeyInfo Canada Post service.
 * <p>
 * The GetNCShipmentPublicKeyInfo service is used to retrieve XML details about the
 * public key.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment. 
 * Use your development key username and password for the web service credentials.
 */

public class GetNCShipmentPublicKeyInfoClient {

    // Main class for local testing only
    public static void main(String[] args) {    	
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_NCShipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            NCShipmentService service = new NCShipmentService();
            NCShipmentPortType port = service.getNCShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/ncshipment/v4");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));
    		
        	//create request
            GetNonContractShipmentPublicKeyInfoRequest request = new GetNonContractShipmentPublicKeyInfoRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
        	request.setShipmentId("522121393172629625");

        	// Execute Request
        	GetNonContractShipmentPublicKeyInfoResponse response = port.getNCShipmentPublicKeyInfo(request);

            // Retrieve values from response object
            if (response.getMessages() == null) {
            	PublicKeyInfoType publicKeyInfoType = response.getNonContractShipmentPublicKeyInfo();
            	System.out.println("Expiry Date: " + publicKeyInfoType.getExpiryDate());
                System.out.println("Url: " + publicKeyInfoType.getUrl());
                if(publicKeyInfoType.getQrCode() != null) {
            		//write QR code to file
            		Path pathToQRCodeImage = Paths.get("qr-code.jpeg");
            		Files.write(pathToQRCodeImage, publicKeyInfoType.getQrCode());
            		System.out.println("QR code image created at: " + pathToQRCodeImage.toAbsolutePath());
            	}
            } else {
                // Assume Error
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }                                    
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
