package ca.canadapost.cpcdp.pickup;

import ca.canadapost.cpcdp.pickup.stub.pickupavailability.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.pickup.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetPickupAvailability Canada Post service.
 * <p>
 * The GetPickupAvailability service identifies if pickup services
 * are available for a given Postal Code. This call also provides information
 * related to the type of pickup available and the cut-off times for pickup service
 * requests.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetPickupAvailabilityClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username, password and customer number are imported from the following file: CPCWS_SOAP_Pickup_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            PickupDomainService service = new PickupDomainService();
            PickupDomainPortType port = service.getPickupDomainPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/ad/soap/pickup/availability");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetPickupAvailabilityRequest request = new GetPickupAvailabilityRequest();
            request.setLocale("FR");
            request.setPostalCode("K2B8J6");

            //execute request
            GetPickupAvailabilityResponse response = port.getPickupAvailability(request);

            //retrieve values from response
            if(response.getMessages() == null){
                System.out.println("Postal Code: " + response.getPickupAvailability().getPostalCode());
                System.out.println("On Demand Tour: " + response.getPickupAvailability().isOnDemandTour());
                System.out.println("On Demand Cut Off: " + response.getPickupAvailability().getOnDemandCutoff());
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
