package ca.canadapost.cpcdp.serviceinfo;

import ca.canadapost.cpcdp.serviceinfo.stub.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.serviceinfo.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetShipmentServiceInfo Canada Post service.
 *<p>
 * Allows you to retrieve and display informational messages for your e-commerce platform customers to inform them
 * about upcoming outages to Canada Post web services.
 * </p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetShipmentServiceInfoClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username and password are imported from the following file: CPCWS_SOAP_Serviceinfo_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");

            ServiceInfoService service = new ServiceInfoService();
            ServiceInfoPortType port = service.getServiceInfoPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/serviceinfo/v2");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            ServiceinfoRequestType request = new ServiceinfoRequestType();
            request.setLocale("FR");

            //execute request
            ServiceinfoResponseType response = port.getShipmentServiceInfo(request);

            //retrieve values from response
            if(response.getMessages() == null){
                if(response.getInfoMessages() != null){
                    for(InfoMessageType message : response.getInfoMessages().getInfoMessage()){
                        System.out.println("Message text: " + message.getMessageText());
                        System.out.println("Message type: " + message.getMessageType());
                        System.out.println("Message Start Date: " + message.getFromDatetime());
                        System.out.println("Message End Date: " + message.getToDatetime());
                    }
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
