package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.util.ClientUtil;
import ca.canadapost.cpcdp.shipping.stub.manifest.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetManifestArtifactId Canada Post service.
 * <p>
 * The GetManifestArtifactId service is used to retrieve the rendered manifest document(s).
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetManifestArtifactIdClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try{

            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ManifestService service = new ManifestService();
            ManifestPortType port = service.getManifestPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/manifest/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetManifestArtifactIdRequest request = new GetManifestArtifactIdRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setManifestId("522121392945877248");

            //execute request
            GetManifestArtifactIdResponse response = port.getManifestArtifactId(request);
            if(response.getMessages() == null){
                System.out.println("PO Number: " + response.getManifest().getPoNumber());
                System.out.println("Artifact Id: " + response.getManifest().getArtifactId());
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}