package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.manifest.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.datatype.DatatypeFactory;
import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetManifests Canada Post service.
 * <p>
 * The GetManifests service is used to retrieve a list of manifests (within
 * a given date range) that were previously transmitted.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetManifestsClient {
    // Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ManifestService service = new ManifestService();
            ManifestPortType port = service.getManifestPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/manifest/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //create request
            GetManifestsRequest request = new GetManifestsRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setStart(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-02-20"));
            request.setEnd(DatatypeFactory.newInstance().newXMLGregorianCalendar("2014-02-21"));

            //execute request
            GetManifestsResponse response = port.getManifests(request);

            //retrieve values from response
            if(response.getMessages() == null){
                if(response.getManifests() != null && !response.getManifests().getManifestId().isEmpty()){
                    for(String id : response.getManifests().getManifestId()){
                        System.out.println("Manifest Id: " + id);
                    }
                }
                else {
                    System.out.println("No manifests returned.");
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }

        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
