package ca.canadapost.cpcdp.shipping;

import ca.canadapost.cpcdp.shipping.stub.customerinfo.*;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;
import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;
import java.util.Map;

/**
 * Sample code for the GetMOBOCustomerInformation Canada Post service.
 * <p>
 * The GetMOBOCustomerInformation service retrieves general information about a Canada Post
 * mailed-on-behalf-of (mobo) customer including contract number, the valid payers and the
 * allowed methods of payment for each payer. (The mobo customer is identified by the
 * mailed-on-behalf-of customer number)
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetMoBoCustomerInformationClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try {
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            CustomerInfoService service = new CustomerInfoService();
            CustomerInfoPortType port = service.getCustomerInfoPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/customerinfo");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetMoboCustomerInformationRequest request = new GetMoboCustomerInformationRequest();
            request.setLocale("FR");
            request.setCustomerNumber(mailedBy);
            request.setMobo(mailedBy);

            //execute request
            GetMoboCustomerInformationResponse response = port.getMoboCustomerInformation(request);
            if(response.getMessages() == null){
                System.out.println("Customer Number: " + response.getBehalfOfCustomer().getCustomerNumber());
                System.out.println();

                System.out.println("Contract Ids:");
                for ( int i = 0; i < response.getBehalfOfCustomer().getContracts().getContractId().size(); i++) {
                    System.out.println("- " + response.getBehalfOfCustomer().getContracts().getContractId().get(i));
                }
                System.out.println();

                System.out.println("Payers:");
                for ( int i = 0; i < response.getBehalfOfCustomer().getAuthorizedPayers().getPayer().size(); i++) {
                    System.out.println("- Customer Number: " + response.getBehalfOfCustomer().getAuthorizedPayers().getPayer().get(i).getPayerNumber());
                    if ( response.getBehalfOfCustomer().getAuthorizedPayers().getPayer().get(i).getMethodsOfPayment() != null) {
                        for ( int j = 0; j < response.getBehalfOfCustomer().getAuthorizedPayers().getPayer().get(i).getMethodsOfPayment().getMethodOfPayment().size(); j++) {
                            if ( j == 0 ) {
                                System.out.println("  Payment Methods:");
                            }
                            System.out.println("  - " + response.getBehalfOfCustomer().getAuthorizedPayers().getPayer().get(i).getMethodsOfPayment().getMethodOfPayment().get(j));
                        }
                    }
                    System.out.println();
                }
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
