package ca.canadapost.cpcdp.shipping;


import ca.canadapost.cpcdp.shipping.stub.shipment.*;
import ca.canadapost.cpcdp.shipping.stub.shipment.ShipmentReceiptType.CcReceiptDetails;
import ca.canadapost.cpcdp.shipping.stub.shipment.ShipmentReceiptType.SupplierAccountReceiptDetails;

import org.apache.cxf.endpoint.Client;
import org.apache.cxf.frontend.ClientProxy;

import ca.canadapost.cpcdp.shipping.util.ClientUtil;

import javax.xml.ws.BindingProvider;
import javax.xml.ws.soap.SOAPFaultException;

import java.text.SimpleDateFormat;
import java.util.Map;

/**
 * Sample code for the GetShipmentReceipt Canada Post service.
 * <p>
 * The GetShipmentReceipt service is used to retrieve the shipment credit card receipt in XML
 * format for a shipment created by a prior no-manifest "create shipment" call with a method of payment CreditCard.
 * <p>
 * This sample is configured to access the Developer Program sandbox environment.
 * Use your development key username and password for the web service credentials.
 */
public class GetShipmentReceiptClient {
	// Main class for local testing only
    public static void main(String[] args) {
        try{
			// Your username and password are imported from the following file: CPCWS_SOAP_Shipping_Java_Samples/user.properties
            Map<String, String> userProperties = ClientUtil.getUserProperties("user.properties");
            String username = userProperties.get("username");
            String password = userProperties.get("password");
            String mailedBy = userProperties.get("mailedBy");

            ShipmentService service = new ShipmentService();
            ShipmentPortType port = service.getShipmentPort();

            //set endpoint
            Map ctx = ((BindingProvider) port).getRequestContext();
            ctx.put(BindingProvider.ENDPOINT_ADDRESS_PROPERTY, "https://ct.soa-gw.canadapost.ca/rs/soap/shipment/v8");

            //wss4j interceptor -  add UsernameToken to soap header
            Client client = ClientProxy.getClient(port);
            client.getOutInterceptors().add(ClientUtil.getUsernameTokenInterceptor(username, password));

            //build request
            GetShipmentReceiptRequest request = new GetShipmentReceiptRequest();
            request.setLocale("FR");
            request.setMailedBy(mailedBy);
            request.setShipmentId("832091378834466920");

            //execute request
            GetShipmentReceiptResponse response = port.getShipmentReceipt(request);

            //retrieve values from response
            if(response.getMessages() == null){
                ShipmentReceiptType shipmentReceiptType = response.getShipmentReceipt();
                if(shipmentReceiptType.getCcReceiptDetails() != null){
                	CcReceiptDetails ccReceiptDetails = shipmentReceiptType.getCcReceiptDetails();
                	System.out.println("Auth Code: " + ccReceiptDetails.getAuthCode());
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                    String authTimestamp = sdf.format(ccReceiptDetails.getAuthTimestamp().toGregorianCalendar().getTime());
                    System.out.println("Auth Timestamp: " + authTimestamp);
                    System.out.println("Charge Amount: " + ccReceiptDetails.getChargeAmount());
                    System.out.println("Currency: " + ccReceiptDetails.getCurrency());
                    System.out.println("Merchant Name: " + ccReceiptDetails.getMerchantName());
	                System.out.println("Merchant URL: " + ccReceiptDetails.getMerchantUrl());
	                System.out.println("Name on Card: " + ccReceiptDetails.getNameOnCard());
	                System.out.println("Card Type: " + ccReceiptDetails.getCardType());
	                System.out.println("Transaction Type: " + ccReceiptDetails.getTransactionType());
                } else if(shipmentReceiptType.getSupplierAccountReceiptDetails() != null){
                	SupplierAccountReceiptDetails supplierAccountReceiptDetails = shipmentReceiptType.getSupplierAccountReceiptDetails();
                	System.out.println("Auth Code: " + supplierAccountReceiptDetails.getAuthCode());
                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
                    String authTimestamp = sdf.format(supplierAccountReceiptDetails.getAuthTimestamp().toGregorianCalendar().getTime());
                    System.out.println("Auth Timestamp: " + authTimestamp);
                    System.out.println("Charge Amount: " + supplierAccountReceiptDetails.getChargeAmount());
                    System.out.println("Currency: " + supplierAccountReceiptDetails.getCurrency());
                    System.out.println("Merchant Name: " + supplierAccountReceiptDetails.getMerchantName());
	                System.out.println("Merchant URL: " + supplierAccountReceiptDetails.getMerchantUrl());
	                System.out.println("Supplier Id: " + supplierAccountReceiptDetails.getSupplierId());
	                System.out.println("Transaction Type: " + supplierAccountReceiptDetails.getTransactionType());
                }
                
            } else {
                for(Messages.Message m : response.getMessages().getMessage()){
                    System.out.println("Error Code: " + m.getCode());
                    System.out.println("Error Message: " + m.getDescription());
                }
            }
        }
        catch (SOAPFaultException e) {
            System.out.println("Fault Code: " + e.getFault().getFaultCode());
            System.out.println("Fault Reason: " + e.getFault().getFaultString());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
